from hydra_agent.v8.connection_string import ConnectionString


class ServerConnectionString(ConnectionString):

    def __init__(self, server, name, user=None, password=None,
                 db_engine=None, db_server=None, db_name=None, db_user=None, db_password=None,
                 date_offset=None, create_if_not_exist=None, locale=None,
                 cluster_user=None, cluster_password=None,
                 license_server_distribution=None, allow_scheduled_jobs=None):
        assert not db_engine or db_engine in ["MSSQLServer", "PostgreSQL", "IBMDB2", "OracleDatabase"]

        self.server = server
        self.name = name
        self.user = user
        self.password = password
        self.db_engine = db_engine
        self.db_server = db_server
        self.db_name = db_name
        self.db_user = db_user
        self.db_password = db_password
        self.date_offset = date_offset
        self._create_if_not_exist = create_if_not_exist
        self.locale = locale
        self.cluster_user = cluster_user
        self.cluster_password = cluster_password
        self._license_server_distribution = license_server_distribution
        self._allow_scheduled_jobs = allow_scheduled_jobs

    @property
    def create_if_not_exist(self):
        return self._bool2str(self._create_if_not_exist)

    @property
    def license_server_distribution(self):
        return self._bool2str(self._license_server_distribution)

    @property
    def allow_scheduled_jobs(self):
        if not isinstance(self._allow_scheduled_jobs, bool):
            return None
        return self._bool2str(not self._allow_scheduled_jobs)

    def _bool2str(self, value):
        if not value:
            return None
        return "Y" if value else "N"

    def _get_properties(self):
        return {
            "server": "Srvr",
            "name": "Ref",
            "user": "Usr",
            "password": "Pwd",
            "db_engine": "DBMS",
            "db_server": "DBSrvr",
            "db_name": "DB",
            "db_user": "DBUID",
            "db_password": "DBPwd",
            "date_offset": "SQLYOffs",
            "create_if_not_exist": "CrSQLDB",
            "locale": "Locale",
            "cluster_user": "SUsr",
            "cluster_password": "SPwd",
            "license_server_distribution": "LicDstr",
            "allow_scheduled_jobs": "SchJobDn",
        }



# This file is part of HYDRA - cross-platform remote administration
# system for 1C:Enterprise (https://github.com/vbondarevsky/hydra_agent).
# Copyright (C) 2017  Vladimir Bondarevskiy.
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


class InfoBase:
    id = ""
    name = ""
    description = ""

    def __init__(self, session_id, cluster):
        self.id = session_id.strip()
        self.cluster = cluster

    def __eq__(self, other):
        return self.__dict__ == other.__dict__

    def __repr__(self):
        return f"<InfoBase(id={self.id}, name={self.name}, cluster={self.cluster})>"

    @staticmethod
    def from_dict(params, cluster):
        infobase = InfoBase(params["infobase"], cluster)
        infobase.name = params["name"]
        infobase.description = params["descr"]
        return infobase

    def to_dict(self):
        return {
            "id": self.id,
            "name": self.name,
            "description": self.description,
            "cluster": self.cluster.to_dict(),
        }



# This file is part of HYDRA - cross-platform remote administration
# system for 1C:Enterprise (https://github.com/vbondarevsky/hydra_agent).
# Copyright (C) 2017  Vladimir Bondarevskiy.
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from hydra_agent import Rac
from hydra_agent import config
from hydra_agent.rac.infobase_manager.infobase import InfoBase


class InfoBaseManager(Rac):
    def __init__(self, cluster, settings=None):
        if not settings:
            settings = config.rac
        super().__init__(settings)
        self.cluster = cluster

    def list(self):
        """Returns collection of `InfoBase`"""

        output = super()._run_command(["infobase", "summary", "list", f"--cluster={self.cluster.id}"])
        for params in self._parse_output(output):
            yield InfoBase.from_dict(params, self.cluster)



# This file is part of HYDRA - cross-platform remote administration
# system for 1C:Enterprise (https://github.com/vbondarevsky/hydra_agent).
# Copyright (C) 2017  Vladimir Bondarevskiy.
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import uuid

from hydra_agent import config
from hydra_agent.utils import is_windows
from hydra_agent.utils.system import run_command_async
from hydra_agent.utils.system import temp_file_name


class V8:
    def __init__(self, connection_string, access_code=None, settings=None):
        self.connection_string = connection_string
        self.access_code = access_code
        if not settings:
            settings = config.v8
        self.path = os.path.join(settings.path, "1cv8" + (".exe" if is_windows() else ""))
        self.display = settings.display

    async def save_db(self, out):
        await self._run_command([
            "DESIGNER",
            "/S", f"{self.connection_string.server}\\{self.connection_string.name}",
            "/N", str(self.connection_string.user),
            "P", str(self.connection_string.password),
            "/DumpIB", out,
            "/DisableSplash",
            "/DisableStartupDialogs",
            "/DisableStartupMessages",
        ])

    async def load_db(self, file_db):
        await self._run_designer(["/RestoreIB", file_db])

    async def create_db(self, name=None, template=None):
        args = [str(self.connection_string)]
        if name:
            args.extend(["/AddToList", name])
        if template:
            args.extend(["/UseTemplate", template])
        await self._run_creation(args)

    async def update_cf(self, file_cf):
        await self._run_designer(["/UpdateCfg", file_cf])

    async def update_db(self):
        await self._run_designer(["/UpdateDBCfg"])

    async def load_extension(self, extension_file):
        extension = "extension_" + uuid.uuid4().hex
        args = [
            "/LoadCfg", extension_file,
            "-Extension", extension,
            "/UpdateDBCfg"
        ]
        await self._run_designer(args)

    async def run_data_processor(self, data_processor):
        await self._run_enterprise(["/Execute", data_processor])

    async def _run_designer(self, args):
        args.extend(["/IBConnectionString", str(self.connection_string)])
        await self._run_command([
            "DESIGNER",
            *args,
            "/DisableSplash",
            "/DisableStartupDialogs",
            "/DisableStartupMessages",
        ])

    async def _run_enterprise(self, args):
        args.extend(["/IBConnectionString", str(self.connection_string)])
        await self._run_command([
            "ENTERPRISE",
            *args,
            "/DisableSplash",
            "/DisableStartupDialogs",
            "/DisableStartupMessages",
        ])

    async def _run_creation(self, args):
        await self._run_command([
            "CREATEINFOBASE",
            *args,
        ])

    async def _run_command(self, args):
        if self.access_code:
            args.extend(["/UC", self.access_code])
        log_file = temp_file_name()
        print(log_file)
        env = os.environ.copy()
        if self.display:
            env["DISPLAY"] = str(self.display)
        result = await run_command_async([self.path, *args, "/Out", log_file], env)
        log = open(log_file).read()
        print(log)
        os.remove(log_file)
        return result



from abc import ABC, abstractmethod


class ConnectionString(ABC):

    @property
    def connection_string(self):
        parts = []
        for k, v in self._get_properties().items():
            if getattr(self, k):
                parts.append(f"{v}='{getattr(self, k)}'")
        return ";".join(parts) + ";"

    @abstractmethod
    def _get_properties(self):
        pass

    def __repr__(self):
        return self.connection_string



# This file is part of HYDRA - cross-platform remote administration
# system for 1C:Enterprise (https://github.com/vbondarevsky/hydra_agent).
# Copyright (C) 2017  Vladimir Bondarevskiy.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os

from hydra_agent import config
from hydra_agent.utils import is_windows
from hydra_agent.utils.system import run_command_async


class PostgreSQL:
    def __init__(self, name, settings=None):
        self.name = name
        if not settings:
            settings = config.db.postgresql
        self.bin = settings.path
        self.user = settings.user
        self.password = settings.password

    def _build_path(self, exe):
        return os.path.join(self.bin, exe + (".exe" if is_windows() else ""))

    @property
    def pg_dump(self):
        return self._build_path("pg_dump")

    @property
    def pg_restore(self):
        return self._build_path("pg_restore")

    @property
    def createdb(self):
        return self._build_path("createdb")

    @property
    def dropdb(self):
        return self._build_path("dropdb")

    async def save(self, file):
        await self._run_command([
            self.pg_dump,
            "-b", "-F", "c", "-Z", "9",
            "-f", file])

    async def load(self, file):
        args = [self.pg_restore]
        if self.user:
            args.extend(["-U", self.user])
        args.extend(["-d", self.name, file])
        await run_command_async(args)

    async def create(self):
        await self._run_command([self.createdb, "-T", "template0"])

    async def remove(self):
        await self._run_command([self.dropdb])

    async def _run_command(self, args):
        if self.user:
            args.extend(["-U", self.user])
        await run_command_async([*args, self.name])



from hydra_agent.v8.connection_string import ConnectionString


class FileConnectionString(ConnectionString):

    def __init__(self, path, user=None, password=None, locale=None, version=None, page_size=None):
        assert not version or version in ["8.2.14", "8.3.8"]
        assert not page_size or page_size in [4096, 8192, 16384, 32768, 65536]

        self.path = path
        self.user = user
        self.password = password
        self.locale = locale
        self.version = version
        self.page_size = page_size

    def _get_properties(self):
        return {
            "path": "File",
            "user": "Usr",
            "password": "Pwd",
            "locale": "Locale",
            "version": "DBFormat",
            "page_size": "DBPageSize",
        }
